import {
  Component,
  OnInit,
  OnDestroy,
  Input,
  Output,
  EventEmitter,
  ViewChild,
  ContentChild,
  TemplateRef
} from '@angular/core';

import {
  VirtualScrollStrategy,
  CdkVirtualScrollViewport
} from '@angular/cdk/scrolling';
import { FlatTreeControl } from '@angular/cdk/tree';

import { MatTreeFlattener, MatTree } from '@angular/material/tree';

import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';

import { VirtualScrollTreeFlatDataSource } from '@ucap/ng-ui';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';

export enum NodeType {
  None = 'None',
  Profile = 'Profile',
  Favorite = 'Favorite',
  Buddy = 'Buddy',
  Default = 'Default'
}

export interface GroupNode {
  nodeType: NodeType;
  userInfo?: UserInfo;
  groupDetail?: GroupDetailData;
  children?: GroupNode[];
}

export interface FlatNode {
  expandable: boolean;
  level: number;
  node: GroupNode;
}

@Component({
  selector: 'ucap-group-expansion-list',
  templateUrl: './expansion-list.component.html',
  styleUrls: ['./expansion-list.component.scss']
})
export class ExpansionListComponent implements OnInit, OnDestroy {
  @Input()
  virtualScrollStrategy: VirtualScrollStrategy;

  @Input()
  set profile(userInfo: UserInfo) {
    if (!userInfo) {
      this.profileNodes = [];
    } else {
      const node: GroupNode = {
        nodeType: NodeType.Profile,
        userInfo,
        children: []
      };

      this.profileNodes = [node];
    }

    this.refreshNodes();
  }

  @Input()
  set favorites(userInfos: UserInfo[]) {
    if (!userInfos || 0 === userInfos.length) {
      this.favoriteNodes = [];
    } else {
      const node: GroupNode = {
        nodeType: NodeType.Favorite,
        groupDetail: {
          seq: -9999,
          name: NodeType.Favorite,
          isActive: true,
          userSeqs: userInfos.map((userInfo) => String(userInfo.seq))
        } as GroupDetailData,
        children: []
      };

      userInfos.forEach((userInfo) => {
        node.children.push({
          nodeType: NodeType.Favorite,
          userInfo
        });
      });

      this.favoriteNodes = [node];
    }

    this.refreshNodes();
  }

  @Input()
  set groupBuddies(list: { group: GroupDetailData; buddyList: UserInfo[] }[]) {
    if (!list || 0 === list.length) {
      this.buddyNodes = [];
    } else {
      for (const item of list) {
        let nodeType = NodeType.Buddy;
        if (0 === item.group.seq) {
          nodeType = NodeType.Default;
        }

        const node: GroupNode = {
          nodeType,
          groupDetail: item.group,
          children: []
        };

        item.buddyList.sort((a, b) =>
          a.order < b.order
            ? -1
            : a.order > b.order
            ? 1
            : a.name < b.name
            ? -1
            : a.name > b.name
            ? 1
            : 0
        );

        item.buddyList.forEach((userInfo) => {
          node.children.push({
            nodeType,
            groupDetail: item.group,
            userInfo
          });
        });

        this.buddyNodes.push(node);
      }
    }

    this.refreshNodes();
  }

  @Input()
  checkable = false;

  @Input()
  selectedUserList?: (UserInfo | UserInfoSS | UserInfoF | UserInfoDN)[] = [];

  @Input()
  unselectableUserList?: (
    | UserInfo
    | UserInfoSS
    | UserInfoF
    | UserInfoDN
  )[] = [];

  @ViewChild('treeList', { static: false })
  treeList: MatTree<FlatNode>;

  @ViewChild('cvsvList', { static: false })
  cvsvList: CdkVirtualScrollViewport;

  @ContentChild('[ucapGroupExpansionList="node"]', { read: TemplateRef })
  nodeTemplate: TemplateRef<any>;

  @ContentChild('[ucapGroupExpansionList="favoriteHeader"]', {
    read: TemplateRef
  })
  favoriteHeaderTemplate: TemplateRef<any>;

  @ContentChild('[ucapGroupExpansionList="defaultHeader"]', {
    read: TemplateRef
  })
  defaultHeaderTemplate: TemplateRef<any>;

  @ContentChild('[ucapGroupExpansionList="buddyHeader"]', { read: TemplateRef })
  buddyHeaderTemplate: TemplateRef<any>;

  rootNodeList: GroupNode[] = [];
  treeControl: FlatTreeControl<FlatNode>;
  treeFlattener: MatTreeFlattener<GroupNode, FlatNode>;
  dataSource: VirtualScrollTreeFlatDataSource<GroupNode, FlatNode>;

  NodeType = NodeType;

  private profileNodes: GroupNode[] = [];
  private favoriteNodes: GroupNode[] = [];
  private buddyNodes: GroupNode[] = [];

  constructor() {
    this.treeControl = new FlatTreeControl<FlatNode>(
      (node) => node.level,
      (node) => node.expandable
    );

    this.treeFlattener = new MatTreeFlattener<GroupNode, FlatNode>(
      (node: GroupNode, level: number) => {
        return {
          expandable: !!node.children && node.children.length > 0,
          level,
          nodeType: node.nodeType,
          node
        };
      },
      (node) => node.level,
      (node) => node.expandable,
      (node) => node.children
    );

    this.dataSource = new VirtualScrollTreeFlatDataSource<GroupNode, FlatNode>(
      this.treeControl,
      this.treeFlattener
    );
  }

  ngOnInit(): void {}

  ngOnDestroy(): void {}

  onClickHeaderMenu(event: MouseEvent, node: FlatNode) {}

  isCheckedGroup(node: FlatNode): boolean {
    const groupDetail = node.node.groupDetail;

    if (!groupDetail || groupDetail === undefined) {
      return false;
    }

    if (groupDetail.userSeqs.length === 0) {
      return false;
    }

    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      let allExist = true;
      groupDetail.userSeqs.some((seq) => {
        if (
          this.selectedUserList.filter((item) => item.seq === seq).length === 0
        ) {
          allExist = false;
          return true;
        }
      });
      return allExist;
    }
    return false;
  }

  isCheckableGroup(node: FlatNode): boolean {
    if (!!this.unselectableUserList && this.unselectableUserList.length > 0) {
      const groupDetail = node.node.groupDetail;
      let allExist = true;
      groupDetail.userSeqs.some((seq) => {
        if (
          this.unselectableUserList.filter((item) => item.seq === seq)
            .length === 0
        ) {
          allExist = false;
          return true;
        }
      });

      if (allExist) {
        return false;
      }
    }

    return true;
  }

  onChangeCheckGroup(value: boolean, node: FlatNode) {}

  isHeader = (_: number, node: FlatNode) =>
    NodeType.Profile !== node.node.nodeType && 0 === node.level;

  private refreshNodes() {
    this.rootNodeList = [
      ...this.profileNodes,
      ...this.favoriteNodes,
      ...this.buddyNodes
    ];
    this.dataSource.data = this.rootNodeList;
  }
}
